/*

   md5c.c 
   MD5 generation core.
   based on RSA Data Security, Inc., MD5 message-digest algorithm

 */



/* 
Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved.

License to copy and use this software is granted provided that it
is identified as the "RSA Data Security, Inc. MD5 Message-Digest
Algorithm" in all material mentioning or referencing this software
or this function.

License is also granted to make and use derivative works provided
that such works are identified as "derived from the RSA Data
Security, Inc. MD5 Message-Digest Algorithm" in all material
mentioning or referencing the derived work.

RSA Data Security, Inc. makes no representations concerning either
the merchantability of this software or the suitability of this
software for any particular purpose. It is provided "as is"
without express or implied warranty of any kind.

These notices must be retained in any copies of any part of this
documentation and/or software.
 */



#include <stdio.h>
#include "./generic.h"




static unsigned char PADDING[64] =
 {
  0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };




void Encode (output, input, len)
unsigned char *output;
unsigned int *input;
unsigned int len;
{
  register i, j;

  for (i = 0, j = 0; j < len; i++, j += 4) {
 output[j] = (unsigned char)(input[i] & 0xff);
 output[j+1] = (unsigned char)((input[i] >> 8) & 0xff);
 output[j+2] = (unsigned char)((input[i] >> 16) & 0xff);
 output[j+3] = (unsigned char)((input[i] >> 24) & 0xff);
  }
}





void Decode (output, input, len)
unsigned int *output;
unsigned char *input;
unsigned int len;
{
  register i, j;

  for (i = 0, j = 0; j < len; i++, j += 4)
 output[i] = ((unsigned int)input[j]) | (((unsigned int)input[j+1]) << 8) |
   (((unsigned int)input[j+2]) << 16) | (((unsigned int)input[j+3]) << 24);
}





void MD5compact (unsigned int *state, unsigned char *block, unsigned int tail)
{
  register unsigned int a,b,c,d;

#ifdef ISBIG_ENDIAN
  unsigned int  x[14];
#else
  unsigned int *x;
#endif

  a = 0x67452301;
  b = 0xefcdab89;
  c = 0x98badcfe;
  d = 0x10325476; 



#ifdef ISBIG_ENDIAN
  Decode ((unsigned int *)x, block, 56);
#else
  x=(unsigned int *)block;
#endif
  	
  /* Round 1 */
  NFF (a, b, c, d, x[0], S11, SS11, 0xd76aa478); /* 1 */
  NFF (d, a, b, c, x[1], S12, SS12, 0xe8c7b756); /* 2 */
  NFF (c, d, a, b, x[2], S13, SS13, 0x242070db); /* 3 */
  NFF (b, c, d, a, x[3], S14, SS14, 0xc1bdceee); /* 4 */
  NFF (a, b, c, d, x[4], S11, SS11, 0xf57c0faf); /* 5 */
  NFF (d, a, b, c, x[5], S12, SS12, 0x4787c62a); /* 6 */
  NFF (c, d, a, b, x[6], S13, SS13, 0xa8304613); /* 7 */
  NFF (b, c, d, a, x[7], S14, SS14, 0xfd469501); /* 8 */
  NFF (a, b, c, d, x[8], S11, SS11, 0x698098d8); /* 9 */
  NFF (d, a, b, c, x[9], S12, SS12, 0x8b44f7af); /* 10 */
  NFF (c, d, a, b, x[10], S13, SS13, 0xffff5bb1); /* 11 */
  NFF (b, c, d, a, x[11], S14, SS14, 0x895cd7be); /* 12 */
  NFF (a, b, c, d, x[12], S11, SS11, 0x6b901122); /* 13 */
  NFF (d, a, b, c, x[13], S12, SS12, 0xfd987193); /* 14 */
  NFF (c, d, a, b, tail, S13, SS13, 0xa679438e); /* 15 */
  NFF0 (b, c, d, a, S14, SS14, 0x49b40821); /* 16 */

 /* Round 2 */
  NGG (a, b, c, d, x[1], S21, SS21, 0xf61e2562); /* 17 */
  NGG (d, a, b, c, x[6], S22, SS22, 0xc040b340); /* 18 */
  NGG (c, d, a, b, x[11], S23, SS23, 0x265e5a51); /* 19 */
  NGG (b, c, d, a, x[0], S24, SS24, 0xe9b6c7aa); /* 20 */
  NGG (a, b, c, d, x[5], S21, SS21, 0xd62f105d); /* 21 */
  NGG (d, a, b, c, x[10], S22,  SS22, 0x2441453); /* 22 */
  NGG0 (c, d, a, b, S23, SS23, 0xd8a1e681); /* 23 */
  NGG (b, c, d, a, x[4], S24, SS24, 0xe7d3fbc8); /* 24 */
  NGG (a, b, c, d, x[9], S21, SS21, 0x21e1cde6); /* 25 */
  NGG (d, a, b, c, tail, S22, SS22, 0xc33707d6); /* 26 */
  NGG (c, d, a, b, x[3], S23, SS23, 0xf4d50d87); /* 27 */
  NGG (b, c, d, a, x[8], S24, SS24, 0x455a14ed); /* 28 */
  NGG (a, b, c, d, x[13], S21, SS21, 0xa9e3e905); /* 29 */
  NGG (d, a, b, c, x[2], S22, SS22, 0xfcefa3f8); /* 30 */
  NGG (c, d, a, b, x[7], S23, SS23, 0x676f02d9); /* 31 */
  NGG (b, c, d, a, x[12], S24, SS24, 0x8d2a4c8a); /* 32 */

  /* Round 3 */
  NHH (a, b, c, d, x[5], S31, SS31, 0xfffa3942); /* 33 */
  NHH (d, a, b, c, x[8], S32, SS32, 0x8771f681); /* 34 */
  NHH (c, d, a, b, x[11], S33, SS33, 0x6d9d6122); /* 35 */
  NHH (b, c, d, a, tail, S34, SS34, 0xfde5380c); /* 36 */
  NHH (a, b, c, d, x[1], S31, SS31, 0xa4beea44); /* 37 */
  NHH (d, a, b, c, x[4], S32, SS32, 0x4bdecfa9); /* 38 */
  NHH (c, d, a, b, x[7], S33, SS33, 0xf6bb4b60); /* 39 */
  NHH (b, c, d, a, x[10], S34, SS34, 0xbebfbc70); /* 40 */
  NHH (a, b, c, d, x[13], S31, SS31, 0x289b7ec6); /* 41 */
  NHH (d, a, b, c, x[0], S32, SS32, 0xeaa127fa); /* 42 */
  NHH (c, d, a, b, x[3], S33, SS33, 0xd4ef3085); /* 43 */
  NHH (b, c, d, a, x[6], S34,  SS34, 0x4881d05); /* 44 */
  NHH (a, b, c, d, x[9], S31, SS31, 0xd9d4d039); /* 45 */
  NHH (d, a, b, c, x[12], S32, SS32, 0xe6db99e5); /* 46 */
  NHH0 (c, d, a, b, S33, SS33, 0x1fa27cf8); /* 47 */
  NHH (b, c, d, a, x[2], S34, SS34, 0xc4ac5665); /* 48 */

  /* Round 4 */
  NII (a, b, c, d, x[0], S41, SS41,0xf4292244); /* 49 */
  NII (d, a, b, c, x[7], S42, SS42, 0x432aff97); /* 50 */
  NII (c, d, a, b, tail, S43, SS43, 0xab9423a7); /* 51 */
  NII (b, c, d, a, x[5], S44, SS44, 0xfc93a039); /* 52 */
  NII (a, b, c, d, x[12], S41, SS41, 0x655b59c3); /* 53 */
  NII (d, a, b, c, x[3], S42, SS42, 0x8f0ccc92); /* 54 */
  NII (c, d, a, b, x[10], S43, SS43, 0xffeff47d); /* 55 */
  NII (b, c, d, a, x[1], S44, SS44, 0x85845dd1); /* 56 */
  NII (a, b, c, d, x[8], S41, SS41, 0x6fa87e4f); /* 57 */
  NII0 (d, a, b, c, S42, SS42, 0xfe2ce6e0); /* 58 */
  NII (c, d, a, b, x[6], S43, SS43, 0xa3014314); /* 59 */
  NII (b, c, d, a, x[13], S44, SS44, 0x4e0811a1); /* 60 */
  NII (a, b, c, d, x[4], S41, SS41, 0xf7537e82); /* 61 */
  NII (d, a, b, c, x[11], S42, SS42, 0xbd3af235); /* 62 */
  NII (c, d, a, b, x[2], S43, SS43, 0x2ad7d2bb); /* 63 */
  NII (b, c, d, a, x[9], S44, SS44, 0xeb86d391); /* 64 */


 
  *state = a+0x67452301;
  *(state+1) = b+0xefcdab89;
  *(state+2) = c+0x98badcfe;
  *(state+3) = d+0x10325476;
}














void MD5intermediate (unsigned int *state, unsigned char *block, unsigned int tail)
{
  register unsigned int a,b,c,d;

#ifdef ISBIG_ENDIAN
  unsigned int  x[7];
#else
  unsigned int *x;
#endif

  a = 0x67452301;
  b = 0xefcdab89;
  c = 0x98badcfe;
  d = 0x10325476; 



#ifdef ISBIG_ENDIAN
  Decode ((unsigned int *)x, block, 28);
#else
  x=(unsigned int *)block;
#endif
  	
  /* Round 1 */
  NFF (a, b, c, d, x[0], S11, SS11, 0xd76aa478); /* 1 */
  NFF (d, a, b, c, x[1], S12, SS12, 0xe8c7b756); /* 2 */
  NFF (c, d, a, b, x[2], S13, SS13, 0x242070db); /* 3 */
  NFF (b, c, d, a, x[3], S14, SS14, 0xc1bdceee); /* 4 */
  NFF (a, b, c, d, x[4], S11, SS11, 0xf57c0faf); /* 5 */
  NFF (d, a, b, c, x[5], S12, SS12, 0x4787c62a); /* 6 */
  NFF (c, d, a, b, x[6], S13, SS13, 0xa8304613); /* 7 */
  NFF0 (b, c, d, a,  S14, SS14, 0xfd469501); /* 8 */
  NFF0 (a, b, c, d, S11, SS11, 0x698098d8); /* 9 */
  NFF0 (d, a, b, c, S12, SS12, 0x8b44f7af); /* 10 */
  NFF0 (c, d, a, b, S13, SS13, 0xffff5bb1); /* 11 */
  NFF0 (b, c, d, a, S14, SS14, 0x895cd7be); /* 12 */
  NFF0 (a, b, c, d, S11, SS11, 0x6b901122); /* 13 */
  NFF0 (d, a, b, c, S12, SS12, 0xfd987193); /* 14 */
  NFF (c, d, a, b, tail, S13, SS13, 0xa679438e); /* 15 */
  NFF0 (b, c, d, a, S14, SS14, 0x49b40821); /* 16 */

 /* Round 2 */
  NGG (a, b, c, d, x[1], S21, SS21, 0xf61e2562); /* 17 */
  NGG (d, a, b, c, x[6], S22, SS22, 0xc040b340); /* 18 */
  NGG0 (c, d, a, b, S23, SS23, 0x265e5a51); /* 19 */
  NGG (b, c, d, a, x[0], S24, SS24, 0xe9b6c7aa); /* 20 */
  NGG (a, b, c, d, x[5], S21, SS21, 0xd62f105d); /* 21 */
  NGG0 (d, a, b, c, S22,  SS22, 0x2441453); /* 22 */
  NGG0 (c, d, a, b, S23, SS23, 0xd8a1e681); /* 23 */
  NGG (b, c, d, a, x[4], S24, SS24, 0xe7d3fbc8); /* 24 */
  NGG0 (a, b, c, d, S21, SS21, 0x21e1cde6); /* 25 */
  NGG (d, a, b, c, tail, S22, SS22, 0xc33707d6); /* 26 */
  NGG (c, d, a, b, x[3], S23, SS23, 0xf4d50d87); /* 27 */
  NGG0 (b, c, d, a, S24, SS24, 0x455a14ed); /* 28 */
  NGG0 (a, b, c, d, S21, SS21, 0xa9e3e905); /* 29 */
  NGG (d, a, b, c, x[2], S22, SS22, 0xfcefa3f8); /* 30 */
  NGG0 (c, d, a, b, S23, SS23, 0x676f02d9); /* 31 */
  NGG0 (b, c, d, a, S24, SS24, 0x8d2a4c8a); /* 32 */

  /* Round 3 */
  NHH (a, b, c, d, x[5], S31, SS31, 0xfffa3942); /* 33 */
  NHH0 (d, a, b, c, S32, SS32, 0x8771f681); /* 34 */
  NHH0 (c, d, a, b, S33, SS33, 0x6d9d6122); /* 35 */
  NHH (b, c, d, a, tail, S34, SS34, 0xfde5380c); /* 36 */
  NHH (a, b, c, d, x[1], S31, SS31, 0xa4beea44); /* 37 */
  NHH (d, a, b, c, x[4], S32, SS32, 0x4bdecfa9); /* 38 */
  NHH0 (c, d, a, b, S33, SS33, 0xf6bb4b60); /* 39 */
  NHH0 (b, c, d, a, S34, SS34, 0xbebfbc70); /* 40 */
  NHH0 (a, b, c, d, S31, SS31, 0x289b7ec6); /* 41 */
  NHH (d, a, b, c, x[0], S32, SS32, 0xeaa127fa); /* 42 */
  NHH (c, d, a, b, x[3], S33, SS33, 0xd4ef3085); /* 43 */
  NHH (b, c, d, a, x[6], S34,  SS34, 0x4881d05); /* 44 */
  NHH0 (a, b, c, d, S31, SS31, 0xd9d4d039); /* 45 */
  NHH0 (d, a, b, c, S32, SS32, 0xe6db99e5); /* 46 */
  NHH0 (c, d, a, b, S33, SS33, 0x1fa27cf8); /* 47 */
  NHH (b, c, d, a, x[2], S34, SS34, 0xc4ac5665); /* 48 */

  /* Round 4 */
  NII (a, b, c, d, x[0], S41, SS41, 0xf4292244); /* 49 */
  NII0 (d, a, b, c, S42, SS42, 0x432aff97); /* 50 */
  NII (c, d, a, b, tail, S43, SS43, 0xab9423a7); /* 51 */
  NII (b, c, d, a, x[5], S44, SS44, 0xfc93a039); /* 52 */
  NII0 (a, b, c, d, S41, SS41, 0x655b59c3); /* 53 */
  NII (d, a, b, c, x[3], S42, SS42, 0x8f0ccc92); /* 54 */
  NII0 (c, d, a, b, S43, SS43, 0xffeff47d); /* 55 */
  NII (b, c, d, a, x[1], S44, SS44, 0x85845dd1); /* 56 */
  NII0 (a, b, c, d, S41, SS41, 0x6fa87e4f); /* 57 */
  NII0 (d, a, b, c, S42, SS42, 0xfe2ce6e0); /* 58 */
  NII (c, d, a, b, x[6], S43, SS43, 0xa3014314); /* 59 */
  NII0 (b, c, d, a, S44, SS44, 0x4e0811a1); /* 60 */
  NII (a, b, c, d, x[4], S41, SS41, 0xf7537e82); /* 61 */
  NII0 (d, a, b, c, S42, SS42, 0xbd3af235); /* 62 */
  NII (c, d, a, b, x[2], S43, SS43, 0x2ad7d2bb); /* 63 */
  NII0 (b, c, d, a, S44, SS44, 0xeb86d391); /* 64 */


 
  *state = a+0x67452301;
  *(state+1) = b+0xefcdab89;
  *(state+2) = c+0x98badcfe;
  *(state+3) = d+0x10325476;
}





/*

  Credit to Simeon pilgrim.
  his reverse engineering of the MD5 algorithm and the resulting function
  make things going faster as twice as with the original implementation.
  In fact the hash is not entirely calculated, the general process could be simplified
  in some cases. 

*/



extern unsigned int fast, fasta, fastb, fastc, fastd;


#ifdef ISBIG_ENDIAN
extern unsigned int x1[3];
#else
extern unsigned int *x1; 
#endif

extern unsigned int working[4];
extern unsigned int tail;




 int MD5Transform(void)
{
  register unsigned int a;
  register unsigned int	b;
  register unsigned int	c;
  register unsigned int	d;



 
  b = 0xefcdab89;
  c = 0x98badcfe;
  d = 0x10325476; 

  a = 0xd76aa477 + x1[0];
  a = NROTATE_LEFT (a, S11, SS11); 
  a += b; 



  NFF (d, a, b, c, x1[1], S12, SS12, 0xe8c7b756); /* 2 */
  NFF (c, d, a, b, x1[2], S13, SS13, 0x242070db); /* 3 */
  NFF0 (b, c, d, a, S14, SS14, 0xc1bdceee); /* 4 */
  NFF0 (a, b, c, d, S11, SS11, 0xf57c0faf); /* 5 */
  NFF0 (d, a, b, c, S12, SS12, 0x4787c62a); /* 6 */
  NFF0 (c, d, a, b, S13, SS13, 0xa8304613); /* 7 */
  NFF0 (b, c, d, a, S14, SS14, 0xfd469501); /* 8 */
  NFF0 (a, b, c, d, S11, SS11, 0x698098d8); /*9 */
  NFF0 (d, a, b, c, S12, SS12, 0x8b44f7af); /* 10 */
  NFF0 (c, d, a, b, S13, SS13, 0xffff5bb1); /* 11 */
  NFF0 (b, c, d, a, S14, SS14, 0x895cd7be); /* 12 */
  NFF0 (a, b, c, d, S11, SS11, 0x6b901122); /* 13 */
  NFF0 (d, a, b, c, S12, SS12, 0xfd987193); /* 14 */
  NFF (c, d, a, b, tail, S13, SS13, 0xa679438e); /* 15 */
  NFF0 (b, c, d, a, S14, SS14, 0x49b40821); /* 16 */
  
  /*
	Round 2
   */
  NGG (a, b, c, d, x1[1], S21, SS21, 0xf61e2562); /* 17 */
  NGG0 (d, a, b, c,  S22, SS22, 0xc040b340); /* 18 */
  NGG0 (c, d, a, b, S23, SS23, 0x265e5a51); /* 19 */
  NGG (b, c, d, a, x1[0], S24, SS24, 0xe9b6c7aa); /* 20 */
  NGG0 (a, b, c, d, S21, SS21, 0xd62f105d); /* 21 */
  NGG0 (d, a, b, c, S22, SS22,  0x2441453); /* 22 */
  NGG0 (c, d, a, b, S23, SS23, 0xd8a1e681); /* 23 */
  NGG0 (b, c, d, a, S24, SS24, 0xe7d3fbc8); /* 24 */
  NGG0 (a, b, c, d, S21, SS21, 0x21e1cde6); /* 25 */
  NGG (d, a, b, c, tail, S22, SS22, 0xc33707d6); /* 26 */
  NGG0 (c, d, a, b, S23, SS23, 0xf4d50d87); /* 27 */
  NGG0 (b, c, d, a, S24, SS24, 0x455a14ed); /* 28 */
  NGG0 (a, b, c, d, S21, SS21, 0xa9e3e905); /* 29 */
  NGG (d, a, b, c, x1[2], S22, SS22, 0xfcefa3f8); /* 30 */
  NGG0 (c, d, a, b, S23, SS23, 0x676f02d9); /* 31 */
  NGG0 (b, c, d, a, S24, SS24, 0x8d2a4c8a); /* 32 */
  
  /*
	Round 3
   */
  NHH0 (a, b, c, d, S31, SS31, 0xfffa3942); /* 33 */
  NHH0 (d, a, b, c, S32, SS32, 0x8771f681); /* 34 */
  NHH0 (c, d, a, b, S33, SS33, 0x6d9d6122); /* 35 */
  NHH (b, c, d, a, tail, S34, SS34, 0xfde5380c); /* 36 */
  NHH (a, b, c, d, x1[1], S31, SS31, 0xa4beea44); /* 37 */
  NHH0 (d, a, b, c, S32, SS32, 0x4bdecfa9); /* 38 */
  NHH0 (c, d, a, b, S33, SS33, 0xf6bb4b60); /* 39 */
  NHH0 (b, c, d, a, S34, SS34, 0xbebfbc70); /* 40 */
  NHH0 (a, b, c, d, S31, SS31, 0x289b7ec6); /* 41 */
  NHH (d, a, b, c, x1[0], S32, SS32, 0xeaa127fa); /* 42 */
  NHH0 (c, d, a, b, S33, SS33, 0xd4ef3085); /* 43 */
  NHH0 (b, c, d, a, S34, SS34,  0x4881d05); /* 44 */
  NHH0 (a, b, c, d, S31, SS31, 0xd9d4d039); /* 45 */

  if (!fast)
    {
          fasta = working[0]; 
 	  fastb = working[1]; 
 	  fastc = working[2]; 
 	  fastd = working[3]; 
	  fast=1;

	  fastb -= x1[1];
	  NRII0 (fastc, fastd, fasta, fastb, S43, SS43, 0xffeff47d); /* 55 */
	  NRII0 (fastd, fasta, fastb, fastc, S42, SS42, 0x8f0ccc92); /* 54 */
	  NRII0 (fasta, fastb, fastc, fastd, S41, SS41, 0x655b59c3); /* 53 */
	  NRII0 (fastb, fastc, fastd, fasta, S44, SS44, 0xfc93a039); /* 52 */
	  NRII0 (fastc, fastd, fasta, fastb, S43, SS43, 0xab9423a7); /* 51 */
	  fastc -= tail; 
	  NRII0 (fastd, fasta, fastb, fastc, S42, SS42, 0x432aff97); /* 50 */
	  NRII0 (fasta, fastb, fastc, fastd, S41, SS41, 0xf4292244); /* 49 */
  
    }

   

 if( fasta != (a+x1[0]))
	  return 0;

  NHH0 (d, a, b, c, S32, SS32, 0xe6db99e5); /* 46 */
  NHH0 (c, d, a, b, S33, SS33, 0x1fa27cf8); /* 47 */
  NHH (b, c, d, a, x1[2], S34, SS34, 0xc4ac5665); /* 48 */



  if((fastb == b) && (fastc == c) && (fastd == d))
	  return 1; 
  
  return 0;
}









int md5_reverse(unsigned int *result, unsigned int* digest, unsigned int *buf)
{
register unsigned int a1,b1,c1,d1;

a1 = *digest;
b1 = *(digest+1);
c1 = *(digest+2);
d1 = *(digest+3);

NRII0 (b1, c1, d1, a1, S44, SS44, 0xeb86d391); 
NRII0 (c1, d1, a1, b1, S43, SS43, 0x2ad7d2bb); 
c1 -= buf[2];
NRII0 (d1, a1, b1, c1, S42, SS42, 0xbd3af235); 
NRII0 (a1, b1, c1, d1, S41, SS41, 0xf7537e82); 
NRII0 (b1, c1, d1, a1, S44, SS44, 0x4e0811a1); 
NRII0 (c1, d1, a1, b1, S43, SS43, 0xa3014314); 
NRII0 (d1, a1, b1, c1, S42, SS42, 0xfe2ce6e0); 
NRII0 (a1, b1, c1, d1, S41, SS41, 0x6fa87e4f); 
NRII0 (b1, c1, d1, a1, S44, SS44, 0x85845dd1); 

*result=a1;
*(result+1)=b1;
*(result+2)=c1;
*(result+3)=d1;	  

return(1);
}





